//----------------------------------------------------------------
//  Copyright (c) by Dynalith Systems Co., Ltd.
//  All right reserved.
//  http://www.dynalith.com
//----------------------------------------------------------------
// bram_ahb.v
//----------------------------------------------------------------
// VERSION: 2009.01.22.
//----------------------------------------------------------------

module fifo (CLK, reset, Data_o, WE_2, ADD_2, ENABLE, CLK_O, Data_i, ADD_1, WE_1, LED_OP, LED_DONE);
// local parameter 
parameter LENGTH_ADD = 32;
parameter LENGTH_DATA = 8;

// STATE
parameter ST_IDLE = 3'b000;
parameter ST_REQ = 3'b001;
parameter ST_OUT = 3'b010;
parameter ST_DONE = 3'b011;
parameter ST_FINISH = 3'b100;
parameter ST_ERR = 3'b101;

// the number of read count
parameter Len = 12;

// input, output
input CLK;
input reset;
output [LENGTH_DATA-1:0] Data_o;
output reg [1:0] WE_2;
output reg [LENGTH_ADD-1:0] ADD_2;
//output reg [9:0] ADD_2;
input ENABLE;

output CLK_O;
input [LENGTH_DATA-1:0] Data_i;
output reg [LENGTH_ADD-1:0] ADD_1;
output reg [1:0] WE_1;
output reg LED_OP;
output reg LED_DONE;

// regs
reg [LENGTH_DATA-1:0] Data_tmp;
reg [LENGTH_ADD-1:0] L_ADD;
reg [1:0] L_WE;
reg [1:0] L_CNT;

reg [2:0] Cur_State;
reg [2:0]  Next_State;
reg ERR;

reg [2:0] ADD_INC;

reg [Len:0] RB;


// clock out
assign CLK_O = CLK;

// Finite State Machine
always@ (posedge CLK or posedge reset)
begin
	if(reset==1'b1)
		Cur_State <= ST_IDLE;
 
	else
		Cur_State <= Next_State;
end

// Decision of next State 
always@*
	begin
	if (RB!= 9'h00) begin
		casex(	{ENABLE,	reset,		ERR,		Cur_State})	
			{1'b0,		1'bx,		1'bx,		3'bxxx}:		Next_State <= ST_IDLE; 
			{1'b1,		1'b1,		1'bx,		3'bxxx}:		Next_State <= ST_IDLE; 
			{1'bx,		1'bx,		1'b1,		3'bxxx}:		Next_State <= ST_ERR; 
			{1'b1,		1'b0,		1'b0,		ST_IDLE}:	Next_State <= ST_REQ; 
			{1'b1,		1'b0,		1'b0,		ST_REQ}:	Next_State <= ST_OUT; 
			{1'b1,		1'b0,		1'b0,		ST_OUT}: 	Next_State <= ST_DONE; 
			{1'b1,		1'b0,		1'b0,		ST_DONE}:	Next_State <= ST_REQ;
			default : begin 
					Next_State <= ST_IDLE; 
				end
		endcase
	end
	else begin
		Next_State <= ST_FINISH;	 
	end
end

/*
always@*
begin
	case (LENGTH_ADD) 
		8 : ADD_INC <= 3'b001;
		16: ADD_INC <= 3'b010;
		32: ADD_INC <= 3'b100;
	endcase
end
*/

always@(posedge CLK)
begin
	case (Cur_State)
		ST_FINISH : begin
			LED_OP <= 1'b0;
			LED_DONE <= 1'b1;
		end
		ST_ERR : begin
			ERR <= 1'b1;
		end
		ST_DONE : begin 
			L_WE <= L_WE +1;
		 	L_CNT <= L_CNT -1'b1;
			if (L_CNT ==2'b00) begin
				L_CNT <= 2'b11;
				L_ADD <= L_ADD+3'b100;
			end
			RB<=RB-1;
			if(RB==0)begin
				LED_OP <= 1'b0;
				LED_DONE <= 1'b1;
			end
			Data_tmp <= Data_i;
 
		end
		ST_OUT : begin
			WE_1 <= L_WE;
			ADD_1 <= L_ADD;
			
		end
		ST_REQ : begin
			LED_OP <= 1'b1;
			WE_2 <= L_WE;
			ADD_2 <= L_ADD;
		end
		ST_IDLE: begin
			L_ADD <= 0;
			L_CNT <= 2'b11;
			L_WE <= 0;

			ADD_1 <= 0;
			ADD_2 <= 0;

//			ADD_INC <= 0;

			WE_1 <= 0;
			WE_2 <= 0;
			RB <= {Len{1'b1}} + 1'b1;
			Data_tmp <= 0;
			
			ERR<=1'b0;
			LED_OP <= 0;
			LED_DONE <= 0;
		end
	endcase
end




//assign Data_o = (Cur_State==ST_OUT)? Data_i : Data_tmp;
assign Data_o = (Cur_State==ST_DONE)? Data_i : Data_tmp;


	

		


endmodule   
  
