//----------------------------------------------------------------
//  Copyright (c) by Dynalith Systems Co., Ltd.
//  All right reserved.
//  http://www.dynalith.com
//----------------------------------------------------------------
// bram_ahb.v
//----------------------------------------------------------------
// VERSION: 2009.01.22.
//----------------------------------------------------------------

module mem_test (EN, HCLK, RESET, T_ADDR, T_DW, T_WE, T_DR, sram_sel, LED_OP, LED_DONE);
input [3:0] EN;
input wire HCLK;
input wire RESET;
input [31:0] T_ADDR;
input [31:0] T_DW;
input [3:0] T_WE;
output wire [31:0] T_DR;

input wire sram_sel;

output wire LED_OP;
output wire LED_DONE;


wire [31:0] sram_add; 
wire [31:0] sram_DATA;
wire [3:0] sram_WE;
wire [3:0] sram_EN;


wire [31:0] to_internal_mem_ADD;
wire [1:0] to_internal_mem_WE;

wire [7:0] to_internal_mem_Data_o;
wire [31:0] to_internal_mem_Data;

wire CLK_O;

wire [1:0] to_bram_WE;
wire [31:0] to_bram_ADD;
wire [31:0] from_bram_Data;
wire [7:0] from_bram_Data_i;

wire [3:0] to_bram_WE_OUT;
wire [3:0] to_internal_mem_WE_OUT;

assign to_internal_mem_Data[7:0] = to_internal_mem_Data_o;
assign to_internal_mem_Data[15:8] = to_internal_mem_Data_o;
assign to_internal_mem_Data[23:16] = to_internal_mem_Data_o;
assign to_internal_mem_Data[31:24] = to_internal_mem_Data_o;


assign  sram_add = sram_sel ?  to_internal_mem_ADD : T_ADDR;
assign	sram_DATA = sram_sel ?  to_internal_mem_Data : T_DW;
assign	sram_WE = sram_sel ?  ~to_internal_mem_WE_OUT : T_WE;
assign	sram_EN = sram_sel ?  ~to_internal_mem_WE_OUT : EN;

reg_sram Ureg_sram_0 (
							.Clock(  HCLK),
							.Address( sram_add[11:2]),
							.Data( sram_DATA[7:0]),
							.Q ( T_DR[7:0]),
							.WE( sram_WE[0] ),
							.EN(sram_EN[0])
							);

reg_sram Ureg_sram_1 (
							.Clock(  HCLK),
							.Address( sram_add[11:2]),
							.Data( sram_DATA[15:8]),
							.Q ( T_DR[15:8]),
							.WE( sram_WE[1] ),
							.EN(sram_EN[1])
							);

reg_sram Ureg_sram_2 (
							.Clock(  HCLK),
							.Address(sram_add[11:2]),
							.Data( sram_DATA[23:16]),
							.Q ( T_DR[23:16]),
							.WE( sram_WE[2] ),
							.EN(sram_EN[2])
							);

reg_sram Ureg_sram_3 (
							.Clock(  HCLK),
							.Address(sram_add[11:2]),
							.Data( sram_DATA[31:24]),
							.Q ( T_DR[31:24]),
							.WE( sram_WE[3] ),
							.EN(sram_EN[3])
							);


fifo U0 (
	.CLK(HCLK), 
	.reset(RESET), 
	.Data_o(to_internal_mem_Data_o), 
	.WE_2(to_bram_WE), 
	.ADD_2(to_bram_ADD), 
	.ENABLE(sram_sel), 
	.CLK_O(CLK_O), 
	.Data_i(from_bram_Data_i), 
	.ADD_1(to_internal_mem_ADD), 
	.WE_1(to_internal_mem_WE ), 
	.LED_OP(LED_OP), 
	.LED_DONE(LED_DONE));


decoder_2_4 decoder0 (.in(to_bram_WE), .out(to_bram_WE_OUT));
decoder_2_4 decoder1 (.in(to_internal_mem_WE), .out(to_internal_mem_WE_OUT));


assign from_bram_Data_i = (~to_bram_WE_OUT[0]) ? from_bram_Data[7:0]:
      			  (~to_bram_WE_OUT[1]) ? from_bram_Data[15:8]:
			  (~to_bram_WE_OUT[2]) ? from_bram_Data[23:16]: from_bram_Data[31:24]; 



blk_mem_8x1024_0
            Ublkmemsp0 (
                   .clka(  HCLK),
                   .ena(   ~to_bram_WE_OUT[0]),
                   .wea(   to_bram_WE_OUT[0]),
                   .addra( to_bram_ADD[11:2]),
                   .dina(   {8{1'b0}}),
                   .douta( from_bram_Data[7:0])
						 );

   blk_mem_8x1024_1
           Ublkmemsp1 (
                   .addra( to_bram_ADD[11:2]),
                   .clka(  HCLK),
                   .dina(  {8{1'b0}}),
                   .douta( from_bram_Data[15:8]),
                   .ena(   ~to_bram_WE_OUT[1]),
                   .wea(   to_bram_WE_OUT[1])
						 );

   blk_mem_8x1024_2
           Ublkmemsp2 (
                   .addra( to_bram_ADD[11:2]),
                   .clka(  HCLK),
                   .dina(  {8{1'b0}}),
                   .douta( from_bram_Data[23:16]),
                   .ena(   ~to_bram_WE_OUT[2]),
                   .wea(   to_bram_WE_OUT[2])
						 );
						 
   blk_mem_8x1024_3
           Ublkmemsp3 (
                   .addra( to_bram_ADD[11:2]),
                   .clka(  HCLK),
                   .dina(  {8{1'b0}}),
                   .douta( from_bram_Data[31:24]),
                   .ena(   ~to_bram_WE_OUT[3]),
                   .wea(   to_bram_WE_OUT[3]));

endmodule
